%{
//      GPPGParser,   ,   gppg
    public ProgrNode root; //     
    public Parser(AbstractScanner<ValueType, LexLocation> scanner) : base(scanner) { }
%}

%output = SimpleYacc.cs

%union { 
			public double dVal; 
			public int iVal; 
			public string sVal; 
			public Node nVal;
			public BlockNode blVal;
			public ProgrNode prVal;
       }

%using System.IO;
%using ProgramTree;

%namespace SimpleParser

%start progr

%token BEGIN END CYCLE SEMICOLON
%token ASSIGN ASSIGNPLUS ASSIGNMINUS ASSIGNMULT
%token <iVal> INTNUM 
%token <sVal> ID
%token PRINT

%type <nVal> expr assign st comp cycl ident pr
%type <blVal> stlist comp
%type <prVal> progr
%type <iVal> assop

%%

progr   : stlist { root = new ProgrNode($1); }
		;

stlist	: st 
			{ 
				$$ = new BlockNode($1 as StatementNode); 
			}
		| stlist SEMICOLON st 
			{ 
				$1.Add($3 as StatementNode); 
				$$ = $1; 
			}
		;

st	: assign { $$ = $1; }
	| comp  { $$ = $1; }
	| cycl  { $$ = $1; }
	| pr { $$ = $1; }
	;

pr	    : PRINT expr {}
	;

ident   : ID { $$ = new IdNode($1); }	
	;

assop	: ASSIGN { $$ = 0; }
		| ASSIGNPLUS { $$ = 1; }
		| ASSIGNMINUS { $$ = 2; }
		| ASSIGNMULT { $$ = 3; }
		;

	
assign 	: ident assop expr { $$ = new AssignNode($1 as IdNode,$3 as ExprNode,$2); }
		;

expr	: ident  { $$ = $1 as IdNode; }
		| INTNUM { $$ = new NumNode($1); }
		;

comp	: BEGIN stlist END { $$ = $2; }
		;

cycl	: CYCLE expr st { $$ = new CycleNode($2 as ExprNode,$3 as StatementNode); }
		;
	
%%

